@@ -1,5 +1,3 @@
-# mypy: ignore-errors
-
 from typing import Any, Callable, Dict, Optional, TYPE_CHECKING
 
 from pytorch_pfn_extras import reporting
@@ -86,7 +84,7 @@ class BestValueTrigger(trigger_module.Trigger):
         self._summary.load_state_dict(to_load['_summary'])
         self._best_value = to_load['_best_value']
 
-    def may_fire(self, iteration, epoch_length):
+    def may_fire(self, iteration: int, epoch_length: int) -> bool:
         return self._interval_trigger.may_fire(iteration, epoch_length)
 
 
